import React, {Component} from 'react';
import Select from 'react-select';
import chroma from 'chroma-js';
import '../index.css';
import './index.css';

require('bootstrap/dist/js/bootstrap');

let ID = function () {
    // Math.random should be unique because of its seeding algorithm.
    // Convert it to base 36 (numbers + letters), and grab the first 9 characters
    // after the decimal.
    return '_' + Math.random().toString(36).substr(2, 9);
};

let getFilterOptions = (options, key) => {
    return options[key].map( item => {
        return {
            value: ID(), label: item
        };
    });
};

let getFilterData = filters => {
    let filterList = filters.map(item => {
        return {
            prop: item.prop,
            options: item.value.map( val => val.label)
        };
    });
    return [].concat(...filterList).filter( (elem, index, self) => {
        return index === self.indexOf(elem);
    });
};

class DataTableFilter extends Component {
    constructor (props) {
        super(props);
        this.toggleFilter = this.toggleFilter.bind(this);
        this.addFilter = this.addFilter.bind(this);
        this.removeFilter = this.removeFilter.bind(this);
        this.onFilterChange = this.onFilterChange.bind(this);
        this.onMenuOpen = this.onMenuOpen.bind(this);
        this.state = {
            filters: [],
            toggleFilter: true,
            currentFilter: 0
        };
    }

    addFilter () {
        let { keys } = this.props;
        let filters = [...this.state.filters];
        filters.push({
            prop: keys[0].prop,
            value: [],
            options: getFilterOptions(this.props.options, keys[0].prop)
        });
        this.setState({ filters });
    }

    onMenuOpen (index) {
        this.setState( (prevState) => {
            return {
                ...prevState,
                currentFilter: index
            }
        });
    }

    changeFilterProp (e, index) {
        let filters = [...this.state.filters];
        filters[index] = {
            ...filters[index],
            prop: e.target.value,
            options: getFilterOptions(this.props.options, e.target.value)
        };
        this.setState({ filters }, () => {
            let f = getFilterData(filters);
            this.props.changeFilter(f);
        });
    }

    removeFilter (i) {
        let filters = [...this.state.filters];
        filters.splice(i, 1);
        this.setState({ filters }, () => {
            let f = getFilterData(filters);
            this.props.changeFilter(f);
        });
    }

    toggleFilter () {
        this.setState( (prevState) => {
            return {
                toggleFilter: prevState.toggleFilter
            }
        });
    }

    onFilterChange (value) {
        let filters = [...this.state.filters];
        filters[this.state.currentFilter].value = value;
        this.setState({ filters }, () => {
            let f = getFilterData(filters);
            this.props.changeFilter(f);
        });
    }

    render () {
        const { baseColor, keys } = this.props;
        const customStyles = {
            control: (styles, state) => ({
                ...styles,
                maxWidth: 230,
                boxShadow: "none",
                borderColor: state.isFocused ? baseColor : '#ced4da',
                '&:hover': {
                    borderColor: baseColor
                }
            }),
            option: (styles, { data, isDisabled, isFocused, isSelected }) => {
                const color = chroma(baseColor);
                return {
                    ...styles,
                    backgroundColor: isDisabled
                        ? null
                        : isSelected ? baseColor : isFocused ? color.alpha(0.1).css() : null,
                    color: isDisabled
                        ? '#ccc'
                        : isSelected
                            ? chroma.contrast(color, 'white') > 2 ? 'white' : 'black'
                            : '#333',
                    cursor: isDisabled ? 'not-allowed' : 'default'
                };
            }
        };
        console.log(this.state.filters);
        return <div className="dt-filter">
                    { this.state.toggleFilter ? <div className="dt-row-filter-content d-flex flex-column">
                            { this.state.filters.map( (filter, index) => {
                                return <div key={`option_${index}`} className="dt-row-filter-row d-flex align-items-start justify-content-center flex-grow-1 mb-1">
                                    <button type="button" className="btn btn-light btn-sm" onClick={() => this.removeFilter(index)}>
                                        <i className="fa fa-times"/>
                                    </button>
                                    <select onChange={e => this.changeFilterProp(e, index)} value={filter.prop} className="custom-select dt-custom-select ml-1 mr-1">
                                        {
                                            keys.map( (item, i) => {
                                                return <option key={`option_${i}`} value={item.prop}>{item.title}</option>
                                            })
                                        }
                                    </select>
                                    <Select
                                        value={filter.value}
                                        closeMenuOnSelect={false}
                                        isMulti
                                        onChange={this.onFilterChange}
                                        onMenuOpen={() => this.onMenuOpen(index)}
                                        options={filter.options}
                                        styles={customStyles}
                                        className="dt-row-filter-select flex-grow-1"
                                        classNamePrefix="dt-filter"
                                    />
                                </div>;
                            })

                            }
                            <button type="button" className="btn btn-light btn-sm" onClick={() => this.addFilter()}>
                                <i className="fa fa-plus" /> Add filter
                            </button>
                        </div> : ''
                    }
                </div>;
    }
}

export default DataTableFilter;