import React, { Component } from 'react';
import ReactDOM from 'react-dom'
import ReactDOMServer from 'react-dom/server';
import PropTypes from 'prop-types';
import _ from 'lodash';
import moment from 'moment';
import Loader from './loader/index';
import Filter from './filter/index';
import './index.css';

let EventEmitter = require('eventemitter3');

class EventEmitterClient {
    constructor() {
        this.eventEmitter = new EventEmitter();
    }
    on(eventName, listener) {
        this.eventEmitter.on(eventName, listener);
    }
    removeEventListener(eventName, listener) {
        this.eventEmitter.removeListener(eventName, listener);
    }
    emit(event, payload, error = false) {
        this.eventEmitter.emit(event, payload, error);
    }
    getEventEmitter() {
        return this.eventEmitter;
    }
}
let eeclient = new EventEmitterClient();

const $ = require('jquery');
window.jQuery = require('jquery');
require('bootstrap/dist/js/bootstrap');
require('datatables.net');
require('datatables.net-colreorder');
require('datatables.net-fixedcolumns');
require('datatables.net-buttons');
require('datatables.net-buttons/js/buttons.colVis.min');
require('datatables.net-buttons/js/buttons.html5.min');
require('datatables.net-select');
require('./editor/dataTables.editor');

const DEFAULT_STRING_CONTENT = 'N/A';
const DEFAULT_NUM_CONTENT = 0;
const DEFAULT_NUM_DATE_CONTENT = 'Never';
const DEFAULT_HTML_CONTENT = '<span></span>';
const DEFAULT_NUM_FMT_CONTENT = 0;
const DEFAULT_HTML_NUM_CONTENT = 0;
const DEFAULT_HTML_NUM_FMT_CONTENT = 0;
const DEFAULT_DATE_CONTENT = 0;
const TYPES_LIST = ['num', 'date', 'num-fmt', 'html-num', 'html-num-fmt', 'html', 'string', 'num-date'];
const DEFAULT_TARGET_ID = 'dt-data-table-container';
const DEFAULT_COLOR = '#1ab394';
const DEFAULT_TOOLTIP_COLOR = '#293846';
const DT_ROW_CHECKBOX = 'dtRowCheckbox';
const DT_ROW_ACTION = 'dtRowAction';
const BULK_ACTION_COLUMN = [{
    prop: DT_ROW_CHECKBOX,
    className: 'dt-select-checkbox no-colvis'
}];
const ROW_ACTION_COLUMN = [{
    prop: DT_ROW_ACTION,
    className: 'dt-action-dropdown no-colvis'
}];

let isEmpty = value => value == null || value === '';

let isEmptyObject = obj => !obj ? true : Object.keys(obj).length === 0 && obj.constructor === Object;

let isEmptyArray = value => !Array.isArray(value) || (Array.isArray(value) && !value.length);

let isColEditable = col => col.hasOwnProperty('editable') && col.editable;

let isColClickable = col => col.hasOwnProperty('clickable') && col.clickable;

let colHasProp = (col, prop) => col.hasOwnProperty(prop);

let getNotHiddenCols = columns => {
    return columns.filter(col => {
        return !col.hasOwnProperty('hidden') || (col.hasOwnProperty('hidden') && !col.hidden);
    });
};

let getCols = (bulkActions, rowActions, columns) => {
    return ((!isEmptyObject(bulkActions) ? BULK_ACTION_COLUMN : []).concat(columns.filter(col => {
        return !col.hasOwnProperty('hidden') || !col.hidden;
    }))).concat((!isEmptyObject(rowActions) ? ROW_ACTION_COLUMN : []));
};

let changeElColor = (first, second, color) => {
    first.find('.dt-button-text')[0].style.backgroundColor = color;
    second.find('.dt-button-text')[0].style.backgroundColor = '';
};

let generateKey = (pre) => {
    return `${ pre }_${ new Date().getTime() }`;
};

let generateUniuqeId = (pre) => {
    return `${ pre || ''}_${Math.random().toString(36).substr(2, 9)}`;
};

let getColumnsOrder = ( columns, hasBulkActions, defaultOrder ) => {
    if (!isEmptyArray(defaultOrder)) {
        const availableIdxs = columns.filter(col => {
            return (!colHasProp(col, 'hidden') || !col.hidden);
        }).map((col, idx) => {
            return (!colHasProp(col, 'visible') || col.visible) ? (hasBulkActions ? idx+1 : idx) : '';
        }).filter(col => {
            return col !== '';
        });
        const colIdx = defaultOrder[0],
              colOrder = defaultOrder[1];
        const colNewIdx = availableIdxs.includes(colIdx) ? (hasBulkActions ? colIdx+1 : colIdx) : availableIdxs[0];
        return [[ colNewIdx , colOrder ]];
    }

    let idx = columns.filter(col => {
        return (!colHasProp(col, 'hidden') || !col.hidden);
    }).findIndex((col) => {
        return (!colHasProp(col, 'visible') || col.visible);
    });
    return [ (hasBulkActions ? idx+1 : idx), "asc" ];
};

let getCellValue = ({prop, render}, row) => {
    if (!isEmpty(prop) && isEmpty(row[prop])) {
        return '';
    } else if (render) {
        return render(row[prop], row);
    } else {
        return row[prop];
    }
};

let getFilterOptions = (keys, data) => {
    let options = {};
    keys.map( key => {
        return options[key.prop] = data.map( d => {
            return d[key.prop] || getDefaultContent(key);
        }).filter( d => {
            return d;
        });
    });
    return options;
};

let getEditableFields = (columns) => {
    return columns.filter( col => {
        return colHasProp(col, 'editable') && col.editable;
    }).map( col => {
        return {
            name: col.prop
        };
    });
};

let findOneEl = (searchData, row, columns) => {
    let arr = [];
    let haystack = [];
    searchData.map(data => {
        let { options, prop} = data;
        arr = arr.concat(options);
        haystack = haystack.concat(Object.entries(row).map( d => {
            let key = d[0];
            let value = d[1];
            if (key === prop) {
                let col = columns.find( col => {
                    return col.prop === prop;
                });
                return row[key] || getDefaultContent(col);
            }
            return value;
        }));
    });
    if (!arr.length) {
        return true;
    } else {
        return arr.every(function (v) {
            return haystack.indexOf(v) >= 0;
        });
    }
};

let parseStrToNum = str => {
    if (!str) {
        return -1;
    }
    let values = str.split(" ");
    let date = moment(0);
    values.forEach(function (val, i) {
        if (i % 2 !== 0) {
            return;
        }
        date.add(+val.trim(), values[i+1]);
    });
    return +date.toDate();
};
let getDefaultContent = col => {
    let { type = 'string', defaultContent } = col;
    const valType = typeof defaultContent;
    switch (type) {
        case 'string':
            return !valType ? DEFAULT_STRING_CONTENT : (valType === 'string' ? defaultContent : DEFAULT_STRING_CONTENT);
        case 'num':
            return !valType ? DEFAULT_NUM_CONTENT : (valType === 'number' ? defaultContent : DEFAULT_NUM_CONTENT);
        case 'date':
            return !valType ? DEFAULT_DATE_CONTENT : (valType === 'number' ? defaultContent : DEFAULT_DATE_CONTENT);
        case 'num-fmt':
            return !valType ? DEFAULT_NUM_FMT_CONTENT : (valType === 'number' ? defaultContent : DEFAULT_NUM_FMT_CONTENT);
        case 'html-num':
            return !valType ? DEFAULT_HTML_NUM_CONTENT : (valType === 'number' ? defaultContent : DEFAULT_HTML_NUM_CONTENT);
        case 'html-num-fmt':
            return !valType ? DEFAULT_HTML_NUM_FMT_CONTENT : (valType === 'number' ? defaultContent : DEFAULT_HTML_NUM_FMT_CONTENT);
        case 'num-date':
            return !valType ? DEFAULT_NUM_DATE_CONTENT : (valType === 'string' ? defaultContent : DEFAULT_NUM_DATE_CONTENT);
        case 'html':
            return !valType ? DEFAULT_HTML_CONTENT : (valType === 'string' ? defaultContent : DEFAULT_HTML_CONTENT);
    }
};

class DataTable extends Component {
    static propTypes = {
        loading: PropTypes.bool.isRequired,
        dataArray: PropTypes.array.isRequired,
        columns: PropTypes.arrayOf(PropTypes.shape({
            title: PropTypes.string.isRequired,
            prop: PropTypes.string.isRequired,
            render: PropTypes.func,
            sortable: PropTypes.bool,
            visibility: PropTypes.bool,
            type: PropTypes.oneOf(TYPES_LIST),
            defaultContent: PropTypes.string,
            width: PropTypes.oneOfType([
                PropTypes.string,
                PropTypes.number
            ]),
            className: PropTypes.oneOfType([
                PropTypes.string,
                PropTypes.func
            ])
        })).isRequired,
        // the id of the div into which the table will render
        targetId: PropTypes.string,
        bulkActions: PropTypes.objectOf(PropTypes.shape({
            title: PropTypes.string.isRequired,
            action: PropTypes.func.isRequired
        })),
        rowActions: PropTypes.objectOf(PropTypes.shape({
            title: PropTypes.string.isRequired,
            action: PropTypes.func.isRequired
        })),
        clickHandler: PropTypes.func,
        editorSaveHandler: PropTypes.func,
        rowId: PropTypes.string,
        initialPageLength: PropTypes.number,
        options: PropTypes.shape({
            doNotAllowFilter: PropTypes.bool,
            doNotAllowSearch: PropTypes.bool,
            doNotAllowExport: PropTypes.bool,
            doNotAllowColvis: PropTypes.bool,
            quickFilter: PropTypes.shape({
                allow: PropTypes.bool,
                condition: PropTypes.shape({
                    key: PropTypes.string.isRequired,
                    value: PropTypes.oneOfType([
                        PropTypes.number,
                        PropTypes.string
                    ]).isRequired
                })
            }),
            doNotAllowInfo: PropTypes.bool,
            doNotAllowPage: PropTypes.bool,
            baseColor: PropTypes.string,
            tooltipBaseColor: PropTypes.string,
            sortBy: PropTypes.array
        })
    };

    constructor (props) {
        super(props);

        this.postRender = this.postRender.bind(this);
        this.getDTMarkup = this.getDTMarkup.bind(this);
        this.changeFilter = this.changeFilter.bind(this);
        this.showRowFilter = this.showRowFilter.bind(this);
        this.handleClickOutsideFilter = this.handleClickOutsideFilter.bind(this);
        this.getTableColumn = this.getTableColumn.bind(this);
    }

    handleClickOutsideFilter (event) {
        if (ReactDOM.findDOMNode(this.refs.dtDataTableFilter) && !ReactDOM.findDOMNode(this.refs.dtDataTableFilter).contains(event.target)) {
            ReactDOM.findDOMNode(this.refs.dtDataTableFilter).style.display = 'none';
        }
    }

    componentDidMount () {
        this.postRender();
        ReactDOM.findDOMNode(this.refs.dtDataTableFilter).style.display = 'none';
        document.addEventListener('mousedown', this.handleClickOutsideFilter);
    }

    componentDidUpdate() {
        this.postRender();
    }

    shouldComponentUpdate (nextProps) {
        // show loader if datatable is reloading
        ReactDOM.findDOMNode(this.refs.dtDataTableLoader).style.display = nextProps.loading ? 'block' : 'none';
        // check if dataArray changes, if so implement componentDidMount code in componentDidUpdate
        const isEqualDataArray = _.isEqual(this.props.dataArray, nextProps.dataArray);
        return !isEqualDataArray;
    }

    showRowFilter () {
        let display = ReactDOM.findDOMNode(this.refs.dtDataTableFilter).style.display;
        if (display === 'block') {
            ReactDOM.findDOMNode(this.refs.dtDataTableFilter).style.display = 'none';
        } else {
            ReactDOM.findDOMNode(this.refs.dtDataTableFilter).style.display = 'block';
        }
    }

    changeFilter (data) {
        eeclient.emit('TABLE_ROW_FILTER', data);
    }

    componentWillUnmount(){
        eeclient.removeListener('TABLE_ROW_FILTER', () => {});
        document.removeEventListener('mousedown', this.handleClickOutsideFilter);
    }

    getTableColumn () {
        let { bulkActions, rowActions, columns, options, rowId } = this.props;
        let { baseColor = DEFAULT_COLOR, tooltipBaseColor = DEFAULT_TOOLTIP_COLOR } = options || {};
        let cols = getCols(bulkActions, rowActions, columns);

        return cols.map( (col, idx) => {
            if (col.prop === DT_ROW_CHECKBOX) {
                return {
                    defaultContent: 'N/A',
                    name: DT_ROW_CHECKBOX,
                    orderable: false,
                    searchable: false,
                    visible: true,
                    width: '42px',
                    className: `select-checkbox ${col.className}`,
                    style: {color: baseColor},
                    render: function(data, type, row, meta) {
                        var checkbox = $("<input/>", {
                            "type": "checkbox"
                        });
                        return checkbox.prop("outerHTML");
                    }
                }
            } if (col.prop === DT_ROW_ACTION) {
                return {
                    defaultContent: 'N/A',
                    name: DT_ROW_ACTION,
                    orderable: false,
                    searchable: false,
                    visible: true,
                    width: '42px',
                    className: col.className,
                    render: function ( data, type, row, meta ) {
                        const _id = row[rowId] || generateUniuqeId('rowaction');
                        let dropId = `dropdown-${col.prop}-${_id}`;
                        return `<div class="dropdown dt-tooltip-container">
                                    <button class="dropdown-toggle" type="button" style="color: ${baseColor}" id="${dropId}" data-toggle="dropdown" aria-haspopup="true" aria-expanded="false"></button>
                                    <span class="dt-tooltip" style="background-color: ${tooltipBaseColor}">Display actions
                                        <span style="color: ${tooltipBaseColor}" class="dt-tooltip-arrow"></span>
                                    </span>
                                    <div class="dropdown-menu" aria-labelledby="${dropId}">
                                        ${Object.entries(rowActions).map(([key, action], idx) => {
                                            return `<div key="${idx}" class="dropdown-item" data-action-key="${key}" data-action-id="${_id}">${action.title}</div>`
                                        }).join(' ')}
                                    </div>
                                </div>`;
                    }
                }
            } else {
                const defVal = getDefaultContent(col);
                return {
                    name: col.prop,
                    type: col.type,
                    defaultContent: defVal,
                    searchable: colHasProp(col, 'searchable') ? col.searchable : true,
                    visible: colHasProp(col, 'visible') ? col.visible : true,
                    data: col.prop,
                    className: `${col.className || ''} ${isColEditable(col) ? 'dt-edit-action' : ''} ${isColClickable(col) ? 'dt-click-action' : ''}`,
                    render: function ( data, type, row, meta ) {
                        const _id = row[rowId] || generateUniuqeId('dtdata');
                        if ( type === 'display' ) {
                            return `<span class="dt-data" data-action-key=${col.prop} data-action-id=${_id}>${getCellValue(col, row) || defVal}</span>
                                        ${isColEditable(col) ? '<i class="fa fa-edit"/>' : '<i class="fa fa-lock"/>'}`;
                        }
                        return data;
                    }
                };
            }
        });
        //to do
        // if (initialPageLength && initialPageLength <= index) {
        //     return null;
        // }
    }

    postRender() {
        const targetId = this.props.targetId || DEFAULT_TARGET_ID;
        $(`#${targetId}`).empty();

        let table = this.getDTMarkup();
        let container = this.refs.dtDataTableDiv;
        let renderedTable = ReactDOMServer.renderToStaticMarkup(table, container);
        $(`#${targetId}`).append(renderedTable);
        let jqueryTable = $(`#${targetId} table`);

        let { doNotAllowSearch, doNotAllowInfo, doNotAllowPage, doNotAllowExport, doNotAllowColvis, doNotAllowFilter, quickFilter, sortBy, baseColor = DEFAULT_COLOR, tooltipBaseColor = DEFAULT_TOOLTIP_COLOR } = this.props.options || {};
        let { rowActions, bulkActions, columns, clickHandler, rowId, dataArray, editorSaveHandler } = this.props;
        let { showRowFilter } = this;
        let tableDomElement = `<"dt-data-table-header d-flex align-items-center"<"buttons flex-grow-1"B>${!doNotAllowSearch ? '<"search"f>' : ''}><"dt-data-table-body"tr><"dt-data-table-footer d-flex"${!doNotAllowInfo ? '<"info d-flex align-items-center flex-grow-1"li>' : ''}${!doNotAllowPage ? '<"page"p>' : ''}>`;
        let rightColReorder = !isEmptyObject(rowActions) ? { fixedColumnsRight: 1 } : {};
        let leftColReorder = !isEmptyObject(bulkActions) ? { fixedColumnsLeft: 1 } : {};

        let leftColFixed = !isEmptyObject(bulkActions) ? { leftColumns: 1 } : { leftColumns: 0 };
        let rightColFixed = !isEmptyObject(rowActions) ? { rightColumns: 1 } : { rightColumns: 0 };

        let order = getColumnsOrder(columns, !isEmptyObject(bulkActions), sortBy);
        const bulkActionsButton = isEmptyObject(bulkActions) ? [] : [{
            text: `<span class="dt-tooltip" style="background-color: ${tooltipBaseColor}" disabled>Bulk actions 
                    <span style="color: ${tooltipBaseColor}" class="dt-tooltip-arrow" /></span>
                   <div class="dropdown">
                      <button id = "bulkActionsButton" data-toggle = "dropdown" disabled><i class = "fa fa-list"/></button>
                      <div class="dropdown-menu" aria-labelledby="bulkActionsButton">
                            ${Object.entries(bulkActions).map(([key, action], idx) => {
                return `<div key=${generateKey(`div-${idx}`)} 
                                        class="dropdown-item" 
                                        data-action-key=${key}>${action.title}</div>`
            }).join(' ')}
                      </div>
                   </div>`,
            className: 'dt-tooltip-container dt-bulk-actions'
        }];

        const { allow, condition } = quickFilter || {};
        const { key, value } = condition || {};

        const quickFilterButton = !allow ? [] : [{
            text: `<span class="dt-button-text">All</span>
                        <span class="dt-tooltip" style="background-color: ${tooltipBaseColor}">Show all items
                        <span style="color: ${tooltipBaseColor}" class="dt-tooltip-arrow" />
                   </span>`,
            className: 'dt-tooltip-container dt-quick-filter dt-all-quick-filter'
        }, {
            text: `<span class="dt-button-text">My</span>
                        <span class="dt-tooltip" style="background-color: ${tooltipBaseColor}">Show my items
                        <span style="color: ${tooltipBaseColor}" class="dt-tooltip-arrow" />
                   </span>`,
            className: 'dt-tooltip-container dt-quick-filter dt-my-quick-filter'
        }];

        const colvisButtons = doNotAllowColvis ? [] : [{
            extend: 'colvis',
            columns: ':not(.no-colvis)',
            className: 'dt-colvis dt-tooltip-container',
            text: `<span class="dt-tooltip" style="background-color: ${tooltipBaseColor}">Edit columns
                        <span style="color: ${tooltipBaseColor}" class="dt-tooltip-arrow" />
                    </span>
                    <i class="fa fa-cogs" style="color: ${baseColor}"/>`
        }];
        const csvButtons = doNotAllowExport ? [] : [{
            extend: 'csvHtml5',
            text: `<span class="dt-tooltip" style="background-color: ${tooltipBaseColor}">Export to CSV
                        <span style="color: ${tooltipBaseColor}" class="dt-tooltip-arrow" />
                  </span>
                  <i class="fa fa-download" style="color: ${baseColor}">`,
            exportOptions: {
                columns: ':not(.no-colvis)'
            },
            className: 'dt-tooltip-container dt-button-csv',
            filename: function () {
                return `*${new Date()}.csv`;
            },
            header: false
        }];

        const rowFilterButton = doNotAllowFilter ? [] : [{
            text: `<span class="dt-tooltip" style="background-color: ${tooltipBaseColor}">Filter list
                        <span style="color: ${tooltipBaseColor}" class="dt-tooltip-arrow" />
                  </span>
                  <i class="fa fa-filter" style="color: ${baseColor}">`,
            className: 'dt-tooltip-container dt-row-filter'
        }];

        let buttons = [...bulkActionsButton, ...colvisButtons, ...rowFilterButton, ...csvButtons, ...quickFilterButton];

        let select = isEmptyObject(bulkActions) ? {} : {
            style: 'multi',
            selector: 'td.dt-select-checkbox'
        };

        $.fn.dataTableExt.oSort['num-date-asc'] = (a,b) => {
            let c = parseStrToNum(a);
            let d = parseStrToNum(b);
            return c < d ? -1 : c > d ? 1 : 0;
        };

        $.fn.dataTableExt.oSort['num-date-desc'] = (a,b) => {
            let c = parseStrToNum(a);
            let d = parseStrToNum(b);
            return c < d ? 1 : c > d ? -1 : 0;
        };


        jqueryTable.DataTable({
            dom: tableDomElement,
            columns: this.getTableColumn(),
            data: dataArray,
            pagingType: 'full_numbers',
            language: {
                lengthMenu: '_MENU_',
                info: 'Showing page _PAGE_ of _PAGES_',
                infoEmpty: 'No records available',
                infoFiltered: ' ',
                search: ' ',
                searchPlaceholder: 'Search'
            },
            responsive: true,
            colReorder: {
                ...leftColReorder,
                ...rightColReorder,
                realtime: false
            },
            scrollX: true,
            fixedColumns:   {
                ...leftColFixed,
                ...rightColFixed
            },
            rowId,
            order,
            createdRow: function (row, data, dataIndex, cells) {
                $(row).addClass('dt-row-tr');
                $(row).find('td.dt-click-action').css('color', baseColor);
                $(row).find('td.dt-select-checkbox').css('color', baseColor);
            },
            initComplete: function () {
                let table = this.api();
                let quickFilterAll = $(this.api().button('.dt-quick-filter.dt-all-quick-filter').node());
                let quickFilterMy = $(this.api().button('.dt-quick-filter.dt-my-quick-filter').node());
                if (quickFilterAll.length && quickFilterMy.length) {
                    if (!$.fn.dataTable.ext.afnQuickFilter || ($.fn.dataTable.ext.afnQuickFilter && $.fn.dataTable.ext.afnQuickFilter === 'all')) {
                        changeElColor(quickFilterAll, quickFilterMy, baseColor);
                    } else {
                        changeElColor(quickFilterMy, quickFilterAll, baseColor);
                    }
                    quickFilterAll.click( () => {
                        $.fn.dataTable.ext.afnQuickFilter = 'all';
                        $.fn.dataTable.ext.search.pop();
                        table.draw();
                        changeElColor(quickFilterAll, quickFilterMy, baseColor);
                    });
                    quickFilterMy.click( () => {
                        $.fn.dataTable.ext.afnQuickFilter = 'my';
                        $.fn.dataTable.ext.search.push( (settings, searchData, index) => {
                            const rowd = dataArray[index];
                            if (rowd[key] && rowd[key] === value) {
                                return true;
                            }
                        });
                        table.draw();
                        $.fn.dataTable.ext.search.pop();
                        changeElColor(quickFilterMy, quickFilterAll, baseColor);
                    });
                    this.on( 'search.dt', function () {
                        $.fn.dataTable.ext.afnQuickFilter = 'all';
                        changeElColor(quickFilterAll, quickFilterMy, baseColor);
                    } );
                    this.on( 'order.dt', function () {
                        $.fn.dataTable.ext.afnQuickFilter = 'all';
                        changeElColor(quickFilterAll, quickFilterMy, baseColor);
                    });
                }

                let headerCheckbox = $($('table thead th.dt-select-checkbox.select-checkbox')[0]);
                headerCheckbox && headerCheckbox.on('click', 'span', function() {
                    if (headerCheckbox.hasClass('selected')) {
                        table.rows().deselect();
                        headerCheckbox.removeClass('selected');
                    } else {
                        table.rows().select();
                        headerCheckbox.addClass('selected');
                    }
                });

                let bulkActionBtn = $(this.api().button('.dt-bulk-actions').node()).find('.dropdown button')[0];
                let bulkActionBtnTooltip = $(this.api().button('.dt-bulk-actions').node()).find('.dt-tooltip')[0];
                table.on('select deselect', function() {
                    if (table.rows({
                        selected: true
                    }).count() !== table.rows().count()) {
                        headerCheckbox.removeClass('selected');
                        $(bulkActionBtnTooltip).attr('disabled', true);
                        bulkActionBtn.disabled = true;
                    } else {
                        headerCheckbox.addClass('selected');
                        $(bulkActionBtnTooltip).attr('disabled', false);
                        bulkActionBtn.disabled = false;
                    }

                    if (!table.rows({
                        selected: true
                    }).count()) {
                        //All rows are deselected
                        bulkActionBtn.style.color = '#bdbebf';
                        $(bulkActionBtnTooltip).attr('disabled', true);
                        bulkActionBtn.disabled = true;
                    } else {
                        bulkActionBtn.style.color = baseColor;
                        $(bulkActionBtnTooltip).attr('disabled', false);
                        bulkActionBtn.disabled = false;
                    }
                });

                let bulkActionButtons = $(this.api().button('.dt-bulk-actions').node()).find('div.dropdown div.dropdown-menu .dropdown-item');
                Array.prototype.forEach.call(bulkActionButtons, (item) => {
                    let dom = $(item);
                    const key = dom[0].attributes['data-action-key'].value;
                    return dom.click(function (ev) {
                        ev.preventDefault();
                        let ids = table.rows('.selected')[0].map( (item) => {
                            return dataArray[item][rowId];
                        });
                        bulkActions[key].action(ids);
                    });
                });

                if (clickHandler && typeof clickHandler === 'function') {
                    this.api().cells('.dt-click-action').every(function () {
                        const cell = this;
                        const node = $(cell.node());
                        const nodeData = node.find('span.dt-data');
                        const id = nodeData[0].getAttribute('data-action-id');
                        const key = nodeData[0].getAttribute('data-action-key');
                        return nodeData.click(function (ev) {
                            ev.preventDefault();
                            clickHandler({
                                id,
                                key
                            });
                        });
                    });
                }
                if (!isEmptyObject(rowActions)) {
                    this.api().cells('.dt-action-dropdown').every(function () {
                        const cell = this;
                        const item = $(cell.node()).find('div.dropdown div.dropdown-menu .dropdown-item');
                        const id = item[0].attributes['data-action-id'].value;
                        const key = item[0].attributes['data-action-key'].value;
                        return item.click(function (ev) {
                            ev.preventDefault();
                            rowActions[key].action(id);
                        });
                    });
                }
                eeclient.on('TABLE_ROW_FILTER', (data) => {
                    if (!data || (data && !data.length)) {
                        $.fn.dataTable.ext.search.pop();
                        table.draw();
                    } else {
                        $.fn.dataTable.ext.search.push( (settings, searchData, index, row) => {
                            if (findOneEl(data, row, columns)) {
                                return true;
                            }
                        });
                        table.draw();
                        $.fn.dataTable.ext.search.pop();
                    }
                });

                let rowFilterBtn = $(this.api().button('.dt-row-filter').node());
                rowFilterBtn.click( (e) => {
                    e.stopPropagation();
                    showRowFilter();
                });

                const fields = getEditableFields(columns);
                if (fields.length) {
                    let editor = new $.fn.dataTable.Editor( {
                        table: `#${targetId} #dt-table`,
                        idSrc: rowId,
                        fields: fields,
                        ajax: function ( method, url, d, successCallback, errorCallback ) {
                            let output = { data: [] };
                            if ( d.action === 'edit' ) {
                                // Update each edited item with the data submitted
                                $.each( d.data, function (id, value) {
                                    output.data.push( value );
                                } );
                            }
                            // Show Editor what has changed
                            successCallback(d.data);
                        }
                    });

                    this.api().cells().every(function () {
                        const cell = this;
                        const node = $(cell.node()).find('i.fa-edit');
                        return node.click(function (ev) {
                            ev.preventDefault();
                            editor.inline( $(this).parent(), {
                                onReturn: function (param) {
                                    param.submit(null, null, function (data) {
                                        editorSaveHandler && editorSaveHandler({obj: data});
                                    });
                                },
                                buttons: {
                                    label: '&gt;',
                                    submit: 'allIfChanged',
                                    fn: function () {
                                        this.submit(null, null, function (data) {
                                            editorSaveHandler && editorSaveHandler({obj: data});
                                        });
                                    }
                                }
                            });
                        });
                    });
                }
            },
            buttons,
            select
        });
    }

    getDTMarkup() {
        let { columns, options, bulkActions, rowActions }  = this.props;
        let { baseColor = DEFAULT_COLOR } = options;
        let cols = getCols(bulkActions, rowActions, columns);

        let headers = cols.map(
            (col, idx) => {
                return <th data-class-name={col.prop} key={generateKey(`th-${idx}`)} className={col.className}>
                    {
                        col.prop === DT_ROW_CHECKBOX ? <span className="dt-select-all" style={{borderColor: baseColor, backgroundColor: baseColor}}/>
                            : col.prop === DT_ROW_ACTION ? ''
                            : <span className="dt-header-cell">{col.title}<span className="dt-sorting" /></span>
                    }
                </th>;
            }
        );

        return (
            <table id="dt-table" className="cell-border" width="100%" cellSpacing="0">
                <thead>
                    <tr className="dt-row-tr">
                        {headers}
                    </tr>
                </thead>
            </table>
        );
    }

    render() {
        const targetId = this.props.targetId || DEFAULT_TARGET_ID;
        let { loading, columns, dataArray, options = {} } = this.props;
        let { baseColor = DEFAULT_COLOR } = options;
        let keys = getNotHiddenCols(columns);
        return (
            <div className="dt-table-root position-relative">
                <Loader ref="dtDataTableLoader"
                        baseColor={baseColor}/>
                <Filter ref="dtDataTableFilter"
                        options={getFilterOptions(keys, dataArray)}
                        keys={keys}
                        baseColor={baseColor}
                        changeFilter={(d) => this.changeFilter(d)}/>
                {!loading ? <div ref="dtDataTableDiv" id={targetId} className="dt-data-table-container" /> : ''}
            </div>
        );
    }
}

export default DataTable;